% Code to generate the forecast error variabce decompositions of the paper 
% by Laudati, Dario and M. Hashem Pesaran (2021). Identifying the Effects 
% of Sanctions on the Iranian Economy Using Newspaper Coverage. CESifo WP9217

clear 
clc

%Load data from the "LP Sanctions Iran Data" folder
load('master_data_quarterly.mat')      %%load dataset

%% Generate rates of change to contruct vector  z
T = size(fx_free,1);
E = 39;      %% number of obs to be excluded (39 obs up to 1988q4)

s = san_on - 0.4*san_off;
ls = lagmatrix(s,1);

%%Generate growth rates as log difference, and their lags
names = {'fx_free' 'm2' 'cpi' 'y' 'gep' 'geq'  'poil' 'yw'};
levels = [fx_free,  m2,  cpi,   y,  gep,  geq,   poil,   yw];

for k = 1:length(names) 
  d.(names{1,k}) = log(levels(:,k)) - lagmatrix(log(levels(:,k)),1);
  ld.(names{1,k}) = lagmatrix(d.(names{1,k}),1) ;
end

%%Long-run interest rates are just in first difference
d.glr = glr - lagmatrix(glr,1);     ld.glr = lagmatrix(d.glr,1);
l2dp = lagmatrix(ld.cpi,1);

AUX1 = struct2cell(d);      AUX1 = AUX1';     D = cell2mat(AUX1);
AUX2 = struct2cell(ld);     AUX2 = AUX2';     LD = cell2mat(AUX2);
clear vars AUX*

all = [s, ls, D, grve, LD, l2dp, lagmatrix(grve,1)];
alltab = array2table(all, 'VariableNames', {'s', 's_lag', 'dfx', 'dm2', 'dp', 'dy', 'dgep', 'dgeq',...
    'dpoil', 'dyw', 'dglr', 'grv', 'dfxl1', 'dm2l1', 'dpl1', 'dyl1', 'dgepl1', ...
    'dgeql1', 'dpoill1', 'dywl1', 'dglrl1', 'dpl2', 'grvel1'});

master = [datamaster, alltab];

n = 3;                          %Number of endogenous variables
ncon = size([d.yw],2) ;         %Number of controls
ntot = n + ncon +1;             %Endogenous + controls + sanctions

%% SVAR estimates
%Model of foreign exchange
model_dfx = 'dfx ~ s + s_lag + dfxl1 + dpl1 + dyl1 + dyw ';
mdl_fx = fitlm(master, model_dfx, 'Exclude', [1:E])

%Model of inflation
model_dp = 'dp ~ s + s_lag + dfx +  dfxl1 +  dpl1 + dyl1 + dpl2 + dyw ';
mdl_dp = fitlm(master, model_dp, 'Exclude', [1:E])

%Model of output
model_dy = 'dy ~ s + s_lag + dp + dfx + dfxl1 + dpl1 + dyl1 + dyw';
mdl_dy = fitlm(master, model_dy, 'Exclude', [1:E])

%AR(1) for sanctions variables
model_s = 's ~ s_lag';
mdl_s = fitlm(master, model_s, 'Exclude', [1:E, T-1:T])
omega = mdl_s.RMSE; 
rho = table2array(mdl_s.Coefficients(2,1));

%Exogenous variables model VAR(1)
z = [d.yw];
z89 = z([E:T-2], :);
model_z = varm(ncon,1);
[est_z, est_SE, ~, est_res] = estimate(model_z, z89);

mdl = {mdl_fx, mdl_dp, mdl_dy};
endog = {'dfx'  'dp' 'dy'};
exog = {'dyw'};
tot = {'dfx' 'dp' 'dy' 'dyw' 's'};


%%Store residuals for each regression to be used later
%Endogenous variables
for k = 1:length(endog) 
  epsilon.(endog{1,k}) = table2array(mdl{k}.Residuals(E+1:T-2,1));
end
%Exogenous variables
for k = 1:length(exog) 
  epsilon.(exog{1,k}) = est_res(:,k);
end
%Sanctions indicator
epsilon.s =  table2array(mdl_s.Residuals(E+1:T-2,1));

%%Shocks equal to 1 SE of the regressions
for k=1:length(exog)
    RMSE.(exog{1,k}) = sqrt(sum(est_res(:,k).^2)/(length(est_res(:,k))-(ncon+1)));
end
RMSE.s = omega;

%% Parameter values to be inserted in the structural model

A0 = eye(n);   A1 = zeros(n,n);        A2 = zeros(n,n); 
C = zeros(n,ncon);

% coefficients for contemporaneous effects
A0(2,1) = - table2array(mdl_dp.Coefficients(4,1));
A0(3,1) = - table2array(mdl_dy.Coefficients(4,1));  A0(3,2) = - table2array(mdl_dy.Coefficients(5,1));

% coefficients for endogenous vars. Lag = 1
A1(1,1) = table2array(mdl_fx.Coefficients(5,1));   A1(1,2) = table2array(mdl_fx.Coefficients(6,1));    A1(1,3) = table2array(mdl_fx.Coefficients(7,1));
A1(2,1) = table2array(mdl_dp.Coefficients(6,1));   A1(2,2) = table2array(mdl_dp.Coefficients(7,1));    A1(2,3) = table2array(mdl_dp.Coefficients(8,1));
A1(3,1) = table2array(mdl_dy.Coefficients(7,1));   A1(3,2) = table2array(mdl_dy.Coefficients(8,1));    A1(3,3) = table2array(mdl_dy.Coefficients(9,1));

% coefficients for endogenous vars. lag=2
A2(2,2) = table2array(mdl_dp.Coefficients(9,1));

aq = zeros(n,1);        azw = zeros(ncon,1);
gamma0 = zeros(n,1);    gamma1 = zeros(n,1);

aq(1,1) = table2array(mdl_fx.Coefficients(1,1));
aq(2,1) = table2array(mdl_dp.Coefficients(1,1));
aq(3,1) = table2array(mdl_dy.Coefficients(1,1));

azw(:,1) = est_z.Constant(:,1);

as = table2array(mdl_s.Coefficients(1,1));

gamma0(1,1) = table2array(mdl_fx.Coefficients(2,1));
gamma0(2,1) = table2array(mdl_dp.Coefficients(2,1));
gamma0(3,1) = table2array(mdl_dy.Coefficients(2,1));

gamma1(1,1) = table2array(mdl_fx.Coefficients(3,1));
gamma1(2,1) = table2array(mdl_dp.Coefficients(3,1));
gamma1(3,1) = table2array(mdl_dy.Coefficients(3,1));

C(1,1:ncon) = table2array(mdl_fx.Coefficients(4,1));
C(2,1:ncon) = table2array(mdl_dp.Coefficients(5,1));
C(3,1:ncon) = table2array(mdl_dy.Coefficients(6,1));

%%standard errors of the regressions
se = [mdl_fx.RMSE; 
     mdl_dp.RMSE; 
     mdl_dy.RMSE;
     RMSE.dyw;
     RMSE.s ];     
 
%%variance-covariance matrix
sigma = eye(ntot).*se.^2;

% Build a single system of matrices
P0 = [A0, -C, -gamma0;
     zeros(ncon, n), eye(ncon), zeros(ncon,1);
     zeros(1,n), zeros(1,ncon), 1];

P1 = [A1, zeros(n, ncon), gamma1;
    zeros(ncon, n), est_z.AR{1,1}, zeros(ncon, 1);
    zeros(1,n), zeros(1, ncon), rho];

P2 = [A2, zeros(n, ncon), zeros(n,1);
    zeros(ncon, ntot);
    zeros(1, ntot)];

a = [aq ; azw; as];

% Reduced form matrices
F1 = P0\P1;         F2 = P0\P2;


%%  Impulse Response Functions
% Loop to create the matrices G0, G1, ...
years = 5;          H = 4*years;       %%Number of quarters

G = cell(H,1);    G{1,1} = eye(ntot);        G{2,1} = F1; 

for i = 1:size(G,1)
    G{i+2,1} = F1*G{i+1,1} + F2*G{i,1};
end


%% Variables shocks

% Endogenous variables
for ind = 1:length(tot)
    shock.(tot{1,ind}) =  se(ind,1);
    IR.(tot{1,ind}) = zeros(ntot,H);
end

% Generate canonical bases of order ntot
eigen = eye(ntot);  e = cell(1, ntot);
for i = 1:ntot
    e{i} = eigen(i,:)';
end

% All impulse responses for all shocks
for ind = 1:length(tot)
    for h = 1:H
        IR.(tot{1,ind})(:,h) = (G{h,1}*(inv(P0))*e{ind})*shock.(tot{1,ind});
    end
end


%% FEVDs

% Denominators
den = cell(H,ntot);

for i = 1:ntot
    for h = 1:H
        den{h,i} = e{i}'*G{h,1}*(inv(P0))*sigma*(inv(P0)')*(G{h,1})'*e{i};
    end
end

den_cumul = cell(h,i);
for i = 1:ntot
    for h = 1:H
        if h == 1
            den_cumul{h,i} = den{h,i};
        else
            den_cumul{h,i} = den_cumul{h-1,i} + den{h,i};
        end
    end
end


%% Numerators
num = zeros(ntot, ntot, H);

%columns are the variables shocked, rows are the variables reacting
for i = 1:ntot
    for j = 1:ntot
        for h = 1:H
            num(i,j,h)= (e{i}'*G{h,1}*(inv(P0))*e{j})^2;
        end
    end
end

num_cum = zeros(ntot, ntot, H);
for i = 1:ntot
    for j = 1:ntot
        for h = 1:H
            if h == 1
                num_cum(i,j,h)= num(i,j,h);
            else
                num_cum(i,j,h)= num(i,j,h) + num_cum(i,j,h-1);
            end
        end
    end
end

for i = 1:ntot
    for j = 1:ntot
        for h = 1:H
            num_cum(i,j,h)= (sigma(j,j))*num_cum(i,j,h);
        end
    end
end

% Result
for h = 1:H
    for i = 1:ntot
        fevd{h,i} = num_cum(i,:,h)/den_cumul{h,i};
    end
end

% Regroup by Variance contribution of variables shocks for each variable
for k = 1:length(tot) 
	FEVD.(tot{1,k}) = cell2mat(fevd(:,k));
end


%change the order of the variables for convenience in the final output
for k = 1:length(tot) 
	FEVD.(tot{1,k}) = [FEVD.(tot{1,k})(:,5) FEVD.(tot{1,k})(:,1:4)];
end

quarter = 0:1:19; quarter = quarter';
dfx = array2table([quarter FEVD.dfx], 'VariableNames', {'Quarter ahead' 's' 'dfx' 'dp' 'dy' 'dyw'});
dp = array2table([quarter FEVD.dp], 'VariableNames', {'Quarter ahead' 's' 'dfx' 'dp' 'dy' 'dyw'});
dy = array2table([quarter FEVD.dy], 'VariableNames', {'Quarter ahead' 's' 'dfx' 'dp' 'dy' 'dyw'});

filename = '../LP Sanctions Iran Results/SVAR results/FEVDs.xlsx';
writetable(dfx, '../LP Sanctions Iran Results/SVAR results/FEVDs.xlsx', 'Sheet', 'FEVD for dfx', 'Range', 'A1:F10');
writetable(dp, '../LP Sanctions Iran Results/SVAR results/FEVDs.xlsx', 'Sheet', 'FEVD for dp', 'Range', 'A1:F10');
writetable(dy, '../LP Sanctions Iran Results/SVAR results/FEVDs.xlsx', 'Sheet', 'FEVD for dy', 'Range', 'A1:F10');


