% Code to generate the value added growth results of the paper by
% Laudati, Dario and M. Hashem Pesaran (2021). Identifying the Effects of
% Sanctions on the Iranian Economy Using Newspaper Coverage. CESifo WP9217

clear 
clc

%% Set-up the problem
%Load data from the "LP Sanctions Iran Data" folder
load('value_added_yearly.mat')

E = 10;  % observarions until 1989
T = size(fx_free,1);

s = san_on - 0.4*san_off;           %%normalized sanctions variable
ls = lagmatrix(s,1);                %%s(t-1)

% Value added shares
sh_va_agr = va_agr./(va_agr + va_mftg + va_serv);
sh_va_mftg = va_mftg./(va_agr + va_mftg + va_serv);
sh_va_serv = va_serv./(va_agr + va_mftg + va_serv);

%%Generate growth rates as log difference
names = {'va_agr' 'va_mftg' 'va_serv' 'poil' 'gep' 'geq' 'yw' 'y_oil' 'y_tur'...
          'sh_va_agr' 'sh_va_mftg' 'sh_va_serv'};
levels = [va_agr, va_mftg, va_serv, poil, gep, geq, yw, y_oil, y_tur, ...
           sh_va_agr sh_va_mftg sh_va_serv];

%%Generate growth rates as log difference, and their lags
for k = 1:length(names) 
  d.(names{1,k}) = log(levels(:,k)) - lagmatrix(log(levels(:,k)),1);
end

%%Long-run interest rates are just in first difference
d.glr = glr - lagmatrix(glr,1);     ld.glr = lagmatrix(d.glr,1);

%%Matrices of rates of change
AUX = struct2cell(d);      AUX = AUX';     D = cell2mat(AUX);
clear vars AUX


%% Generate regressions estimates for Value added growth
%Estimates Set 1
endog = [d.va_agr d.va_mftg d.va_serv];
endog(1:E-1,:) = [];
exog1 = [s grve d.poil];
exog1(1:E-1,:) = [];
model_va1 = varm(3,1);
[est_coeff_va1, est_SE_va1] = estimate(model_va1, endog, 'X', exog1);


%Estimates Set 2
exog2 = [s grve d.poil d.glr d.gep d.geq];
exog2(1:E-1,:) = [];
model_va2 = varm(3,1);
[est_coeff_va2, est_SE_va2] = estimate(model_va2, endog, 'X', exog2);


%Estimates Set 3
exog3 = [s grve d.poil d.glr d.gep d.geq d.yw];
exog3(1:E-1,:) = [];
model_va3 = varm(3,1);
[est_coeff_va3, est_SE_va3] = estimate(model_va3, endog, 'X', exog3);


%Estimates Set 4
exog4 = [s grve d.poil d.glr d.gep d.geq d.yw d.y_oil d.y_tur];
exog4(1:E-1,:) = [];
model_va4 = varm(3,1);
[est_coeff_va4, est_SE_va4] = estimate(model_va4, endog, 'X', exog4);


%% Generate regressions estimates for sectoral value added shares growth rates

%%Agriculture and Manufacturing shares
%Estimates Set 1
sh_agr_mftg = [d.sh_va_agr d.sh_va_mftg];
sh_agr_mftg(1:E-1,:) = [];
agr_mftg_model1 = varm(2,1);
[est_coeff_agr_mftg1, est_SE_agr_mftg1] = estimate(agr_mftg_model1, sh_agr_mftg, 'X', s);

%Estimates Set 2
agr_mftg_exog2 = [s d.yw];
agr_mftg_model2 = varm(2,1);
[est_coeff_agr_mftg2, est_SE_agr_mftg2] = estimate(agr_mftg_model2, sh_agr_mftg, 'X', agr_mftg_exog2);

%Estimates Set 3
agr_mftg_exog3 = [s d.yw d.poil];
agr_mftg_model3 = varm(2,1);
[est_coeff_agr_mftg3, est_SE_agr_mftg3] = estimate(agr_mftg_model3, sh_agr_mftg, 'X', agr_mftg_exog3);

%Estimates Set 4
agr_mftg_exog4 = [s d.yw d.poil d.y_oil d.y_tur];
agr_mftg_model4 = varm(2,1);
[est_coeff_agr_mftg4, est_SE_agr_mftg4] = estimate(agr_mftg_model4, sh_agr_mftg, 'X', agr_mftg_exog4);

%Estimates Set 5
agr_mftg_exog5 = [s d.yw d.poil d.y_oil d.y_tur grve d.glr d.gep d.geq];
agr_mftg_model5 = varm(2,1);
[est_coeff_agr_mftg5, est_SE_agr_mftg5] = estimate(agr_mftg_model5, sh_agr_mftg, 'X', agr_mftg_exog5);



%% Agriculture and Services shares
%Estimates Set 1
sh_agr_serv= [d.sh_va_agr d.sh_va_serv];
sh_agr_serv(1:E-1,:) = [];
agr_serv_model1 = varm(2,1);
[est_coeff_agr_serv1, est_SE_agr_serv1] = estimate(agr_serv_model1, sh_agr_serv, 'X', s);

%Estimates Set 2
agr_serv_exog2 = [s d.yw];
agr_serv_model2 = varm(2,1);
[est_coeff_agr_serv2, est_SE_agr_serv2] = estimate(agr_serv_model2, sh_agr_serv, 'X', agr_serv_exog2);

%Estimates Set 3
agr_serv_exog3 = [s d.yw d.poil];
agr_serv_model3 = varm(2,1);
[est_coeff_agr_serv3, est_SE_agr_serv3] = estimate(agr_serv_model3, sh_agr_serv, 'X', agr_serv_exog3);

%Estimates Set 4
agr_serv_exog4 = [s d.yw d.poil d.y_oil d.y_tur];
agr_serv_model4 = varm(2,1);
[est_coeff_agr_serv4, est_SE_agr_serv4] = estimate(agr_serv_model4, sh_agr_serv, 'X', agr_serv_exog4);

%Estimates Set 5
agr_serv_exog5 = [s d.yw d.poil d.y_oil d.y_tur grve d.glr d.gep d.geq];
agr_serv_model5 = varm(2,1);
[est_coeff_agr_serv5, est_SE_agr_serv5] = estimate(agr_serv_model5, sh_agr_serv, 'X', agr_serv_exog5);


%% Manufacturing and Services shares
%Estimates Set 1
sh_mftg_serv= [d.sh_va_mftg d.sh_va_serv];
sh_mftg_serv(1:E-1,:) = [];
mftg_serv_model1 = varm(2,1);
[est_coeff_mftg_serv1, est_SE_mftg_serv1] = estimate(mftg_serv_model1, sh_mftg_serv, 'X', s);

%Estimates Set 2
mftg_serv_exog2 = [s d.yw];
mftg_serv_model2 = varm(2,1);
[est_coeff_mftg_serv2, est_SE_mftg_serv2] = estimate(mftg_serv_model2, sh_mftg_serv, 'X', mftg_serv_exog2);

%Estimates Set 3
mftg_serv_exog3 = [s d.yw d.poil];
mftg_serv_model3 = varm(2,1);
[est_coeff_mftg_serv3, est_SE_mftg_serv3] = estimate(mftg_serv_model3, sh_mftg_serv, 'X', mftg_serv_exog3);

%Estimates Set 4
mftg_serv_exog4 = [s d.yw d.poil d.y_oil d.y_tur];
mftg_serv_model4 = varm(2,1);
[est_coeff_mftg_serv4, est_SE_mftg_serv4] = estimate(mftg_serv_model4, sh_mftg_serv, 'X', mftg_serv_exog4);

%Estimates Set 5
mftg_serv_exog5 = [s d.yw d.poil d.y_oil d.y_tur grve d.glr d.gep d.geq];
mftg_serv_model5 = varm(2,1);
[est_coeff_mftg_serv5, est_SE_mftg_serv5] = estimate(mftg_serv_model5, sh_mftg_serv, 'X', mftg_serv_exog5);

save('../LP Sanctions Iran Results/sectoral value added VARX results/sectoral_varx.mat');